Add 'w' suffix for __ibm128 constants.

In the documentation, we mention that 'w' or 'W' can be used as a suffix for
__ibm128 constants.  We never implemented this.  This patch fixes that.

In addition, the 'q' and 'Q' suffix were changed to use the mode used for the
__float128 type, instead of knowing whether to use KFmode or TFmode explicitly.
This will be used in a future patch where we change the mode used for __float128
on systems where long double is IEEE 128-bit.

I have tested this patch on the following systems:

    1)  LE Power10 using --with-cpu=power10 --with-long-double-format=ieee
    2)  LE Power10 using --with-cpu=power9  --with-long-double-format=ibm
    3)  LE Power10 using --with-cpu=power8  --with-long-double-format=ibm
    4)  LE Power10 using --with-cpu=power10 --with-long-double-format=ibm
    5)  LE Power9  using --with-cpu=power9  --with-long-double-format=ibm
    6)  BE Power8  using --with-cpu=power8  --with-long-double-format=ibm
    7)  BE Power8  using --with-cpu=power5  --with-long-double-format=ibm

There were no regressions in the build or in the tests.

Can I check this patch into the trunk?

Did we want to backport this to earlier GCC releases?

2022-08-17   Michael Meissner  <meiss...@linux.ibm.com>

gcc/

        * config/rs6000/rs6000.cc (rs6000_c_mode_for_suffix): Allow 'w' or 'W'
        for __ibm128 constants.

gcc/testsuite/

        * gcc.target/powerpc/ibm128-suffix.c: New test.
---
 gcc/config/rs6000/rs6000.cc                   | 25 ++++++++++---------
 .../gcc.target/powerpc/ibm128-suffix.c        | 13 ++++++++++
 2 files changed, 26 insertions(+), 12 deletions(-)
 create mode 100644 gcc/testsuite/gcc.target/powerpc/ibm128-suffix.c

diff --git a/gcc/testsuite/gcc.target/powerpc/ibm128-suffix.c 
b/gcc/testsuite/gcc.target/powerpc/ibm128-suffix.c
new file mode 100644
index 00000000000..ff619860409
--- /dev/null
+++ b/gcc/testsuite/gcc.target/powerpc/ibm128-suffix.c
@@ -0,0 +1,13 @@
+/* { dg-do compile } */
+/* { dg-require-effective-target longdouble128 } */
+/* { dg-options "-O2" } */
+
+/* See if the 'w' suffix is accepted for __ibm128.  */
+#ifndef NUMBER
+#define NUMBER  123456789012345678901234567890123456789E-10
+#endif
+
+#define GLUE2(X,Y)      X ## Y
+#define GLUE(X,Y)       GLUE2(X,Y)
+
+__ibm128 x = GLUE (NUMBER, w);
diff --git a/gcc/config/rs6000/rs6000.cc b/gcc/config/rs6000/rs6000.cc
index a6ec4c71ac0..046c538c748 100644
--- a/gcc/config/rs6000/rs6000.cc
+++ b/gcc/config/rs6000/rs6000.cc
@@ -23845,22 +23845,23 @@ rs6000_floatn_mode (int n, bool extended)
 
 }
 
-/* Target hook for c_mode_for_suffix.  */
+/* Target hook for c_mode_for_suffix.  We use TYPE_MODE to follow the mode used
+   for __float128 and __ibm128.
+
+   Only two suffixes are allowed, 'q' and 'w'.  The 'q' suffix is used for
+   float128 constants on both the x86 and PowerPC processors.  Its use predates
+   the use of 'f128' for _Float128 constants, and existing code still uses it.
+
+   The 'w' suffix was used on the x86 processors for their 80-bit long
+   double.  We use it for __ibm128 constants.  */
 static machine_mode
 rs6000_c_mode_for_suffix (char suffix)
 {
-  if (TARGET_FLOAT128_TYPE)
-    {
-      if (suffix == 'q' || suffix == 'Q')
-       return (FLOAT128_IEEE_P (TFmode)) ? TFmode : KFmode;
+  if (TARGET_FLOAT128_TYPE && (suffix == 'q' || suffix == 'Q'))
+    return TYPE_MODE (ieee128_float_type_node);
 
-      /* At the moment, we are not defining a suffix for IBM extended double.
-        If/when the default for -mabi=ieeelongdouble is changed, and we want
-        to support __ibm128 constants in legacy library code, we may need to
-        re-evalaute this decision.  Currently, c-lex.cc only supports 'w' and
-        'q' as machine dependent suffixes.  The x86_64 port uses 'w' for
-        __float80 constants.  */
-    }
+  if (TARGET_IBM128 && (suffix == 'w' || suffix == 'W'))
+    return TYPE_MODE (ibm128_float_type_node);
 
   return VOIDmode;
 }
-- 
2.37.2


-- 
Michael Meissner, IBM
PO Box 98, Ayer, Massachusetts, USA, 01432
email: meiss...@linux.ibm.com

Reply via email to